@@ -1,5 +1,4 @@
 from hcat.detect import _detect
-from hcat.segment import _segment
 from hcat.detect_gui import gui
 
 import glob
@@ -19,33 +18,7 @@ This is the main entry point to all analysis scripts. Each analysis script is fo
 def cli(ctx):
     if ctx.invoked_subcommand is None:
         """ Default subcommand is running full analysis """
-        gui()
-
-
-@cli.command()
-@click.argument('f', default=None)
-@click.option('--channel', default=2, help='Channel index to segment')
-@click.option('--intensity_reject_threshold', default=0.05, help='Cell cytosol intensity rejection threshold')
-@click.option('--dtype', default=None, help='dtype of input image')
-@click.option('--unet', is_flag=True, help='Run with Unet+Watershed Backbone')
-@click.option('--cellpose', is_flag=True, help='Run with Cellpose Backbone')
-@click.option('--figure', is_flag=True, help='Save preliminary analysis figure')
-@click.option('--no_post', is_flag=True, help='Do not postprocess')
-def segment(f: str, channel: int, intensity_reject_threshold: float,
-            dtype: Optional[str],
-            unet: bool, cellpose: bool, no_post: bool,
-            figure: Optional[bool]) -> None:
-
-    files = glob.glob(f)
-    for filename in files:
-        try:
-            _segment(f=filename, channel=channel, intensity_reject_threshold=intensity_reject_threshold,
-                     dtype=dtype, unet=unet, cellpose=cellpose, no_post=no_post, figure=figure)
-        except Exception:
-            print(f'Error in file: {filename}')
-
-    return None
-
+        gui().main_loop()
 
 @cli.command()
 @click.argument('f', default=None)
@@ -59,12 +32,16 @@ def segment(f: str, channel: int, intensity_reject_threshold: float,
 @click.option('--normalize', is_flag=True, help='Threshold (between 0 and 1) of cell detection.')
 @click.option('--pixel_size', default=None, help='Pixel size in nm')
 @click.option('--cell_diameter', default=None, help='Cell diameter in pixels')
+@click.option('--predict_curvature', default=None, help='Cell diameter in pixels')
+@click.option('--silent', default=False, help="Suppresses most of HCAT's logging ")
 def detect(f: str, curve_path, cell_detection_threshold, nms_threshold, save_xml, save_png,
-           save_fig, normalize, pixel_size, dtype, cell_diameter):
+           save_fig, normalize, pixel_size, dtype, cell_diameter, predict_curvature, silent):
 
     cell_diameter = float(cell_diameter) if cell_diameter is not None else None
     # Evaluate a single image
 
+    verbose = not silent
+
     files = glob.glob(f)
     for filename in files:
         try:
@@ -78,7 +55,10 @@ def detect(f: str, curve_path, cell_detection_threshold, nms_threshold, save_xml
                     normalize=normalize,
                     pixel_size=pixel_size,
                     cell_diameter=cell_diameter,
-                    dtype=dtype)
+                    dtype=dtype,
+                    predict_curve=predict_curvature,
+                    verbose=verbose)
+
         except Exception as e:
             print(f'Critical Error! Aborting - {filename}')
             raise e
